package org.beginningee6.tutorial;

import javax.annotation.PostConstruct;
import javax.ejb.Singleton;
import javax.ejb.Startup;
import javax.persistence.EntityManager;
import javax.persistence.PersistenceContext;
import java.util.logging.Logger;

/**
 * @author Antonio Goncalves & Alexis Moussine-Pouchkine
 *         Tutorial - Beginning with The Java EE 6 Platform
 *         http://www.antoniogoncalves.org
 *         http://blogs.sun.com/alexismp
 *         --
 *         A startup singleton.
 *         Assumes we're using a drop-and-create-tables DDL option.
 */
@Singleton
@Startup
public class DBPopulator {

    // ======================================
    // =             Attributes             =
    // ======================================

    @PersistenceContext
    private EntityManager em;

    private Logger logger = Logger.getLogger("org.beginningee6.tutorial");

    // ======================================
    // =          Lifecycle methods         =
    // ======================================
    
    @PostConstruct
    private void createDummyData() {
        createBooks();
        createCDs();
    }
    
    // ======================================
    // =          Business methods          =
    // ======================================

    private void createBooks() {
        em.persist(new Book("The Hitchhiker's Guide to the Galaxy", "EN", new Float(23.99), "Science fiction comedy book", "1-84023-742-2", new Integer(354), Boolean.TRUE));
        em.persist(new Book("Harry Potter and the Goblet of Fire", "EN", new Float(19.79), "Science fiction (Book 4)", "1-978-0439139595", new Integer(734), Boolean.TRUE));
        em.persist(new Book("The Elements of Style","EN", new Float(6.64), "A masterpiece in the art of clear and concise writing.", "1-978-0205309023", new Integer(105), Boolean.FALSE));
        em.persist(new Book("Harry Potter And The Order Of The Phoenix","EN", new Float(9.35), "Science fiction (Book 1)", "1-978-0439358071", new Integer(870), Boolean.TRUE));
        em.persist(new Book("The Difference Between God and Larry Ellison","EN", new Float(11.99), "God Doesn't Think He's Larry Ellison", "1-978-0060008765", new Integer(420), Boolean.FALSE));
        em.persist(new Book("The Children of Hurin","EN", new Float(17.16), "The first complete book by J.R.R. Tolkien in three decades", "1-978-0618894642", new Integer(313), Boolean.TRUE));
        em.persist(new Book("The Da Vinci Code","EN", new Float(17.79), "Thriller", "1-978-0385504201", new Integer(454), Boolean.FALSE));
        em.persist(new Book("La carte et le territoire","FR", new Float(24.99), "Prix Goncourt 2010", "1-978-2081246331", new Integer(428), Boolean.FALSE));
        em.persist(new Book("Une forme de vie","FR", new Float(18.99), "Amelie Nothomb", "1-978-2226215178", new Integer(168), Boolean.FALSE));
        em.persist(new Book("To Kill a Mockingbird","EN", new Float(5.99), "American classic", "1-978-0446310789", new Integer(281), Boolean.FALSE));
        em.persist(new Book("Fahrenheit 451","EN", new Float(7.99), "Science fiction", "1-978-0345342966", new Integer(208), Boolean.FALSE));
        em.persist(new Book("Lolita","RU", new Float(14.99), "Nabovok masterpiece", "1-978-0141023496", new Integer(368), Boolean.TRUE));
        em.persist(new Book("A Midsummer Night's Dream","EN", new Float(14.99), "Shakespeare masterpiece", "1-978-0199535866", new Integer(288), Boolean.TRUE));
        em.persist(new Book("I bastioni del coraggio","IT", new Float(18.99), "Anno domini 1548. Una grande storia d'amore, ...", "1-978-88045844", new Integer(457), Boolean.FALSE));
        em.persist(new Book("Harjunpää ja pahan pappi","FI", new Float(32.49), "Finns read crime novels", "1-978-951-1-2-4742-5", new Integer(300), Boolean.FALSE));
        em.persist(new Book("El ingenioso hidalgo don Quijote de la Mancha","ES", new Float(13.99), "Classical Don Quijote", "1-842-4116194", new Integer(108), Boolean.TRUE));
        em.persist(new Book("The Lord of the Rings","EN", new Float(50.4), "Science fiction comedy book", "1-978-061-8-2-6058-4", new Integer(1216), Boolean.TRUE));
        em.persist(new Book("Java EE 6 with GlassFish 3","EN", new Float(31.49), "Java!", "1-978-1430228899", new Integer(354), Boolean.TRUE));
        em.persist(new Book("Heidis Lehr- und Wanderjahre","DE", new Float(68), "Inhalt: Die Heidi-Bücher erzählen...", "1-978-1452826226", new Integer(160), Boolean.TRUE));
        em.persist(new Book("l Nome della Rosa","IT", new Float(34.99), "Science fiction comedy book", "1-884-5214214", new Integer(354), Boolean.FALSE));
        logger.info("&&&&&&&&&&&&&& Inserted " + em.createNamedQuery("findAllBooks").getResultList().size() + " Books");
    }

    private void createCDs() {
        em.persist(new CD("Genesis", (float)12.99, "Genesis are an English rock band that formed in 1967", "Atlantic", 1, (float)44.10));
        em.persist(new CD("The Dark Side of the Moon", (float)14.99, "Pink Floyd were an English rock band", "EMI", 1, (float)42.59));
        em.persist(new CD("Minor Swing", (float)1.99, "Pioneering virtuoso jazz guitarist and composer", "None", 1, (float)3.18));
        em.persist(new CD("Simon and Garfunkel in Central Park", (float)23.99, "Live album recorded on September 19, 1981", "Warner Bros.", 2, (float)75.51));
        em.persist(new CD("Blue Monday", (float)1.99, "New Order's longest tracks ever to chart", "Factory", 1, (float)7.29));
        em.persist(new CD("The Unforgettable Fire", (float)13.99, "The Unforgettable Fire is the fourth studio album by Irish rock band U2", "Island", 1, (float)42.38));
        em.persist(new CD("Synchronicity", (float)15.99, "Fifth and final studio album by The Police, released in 1983", "A&M", 1, (float)44.32));
        em.persist(new CD("Bireli Lagrene & Gypsy Project", (float)16.98, "has perhaps inherited more of Django's musical spirit", "Dreyfus", 2, (float)58.12));
        logger.info("&&&&&&&&&&&&&& Inserted " + em.createNamedQuery("findAllCDs").getResultList().size() + " CDs");
    }
    
}
