package org.beginningee6.tutorial;

import javax.annotation.PostConstruct;
import javax.ejb.EJB;
import javax.enterprise.context.Conversation;
import javax.enterprise.context.ConversationScoped;
import javax.faces.application.FacesMessage;
import javax.faces.context.FacesContext;
import javax.faces.event.ActionEvent;
import javax.inject.Inject;
import javax.inject.Named;
import javax.validation.constraints.Size;
import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;
import java.util.StringTokenizer;
import java.util.logging.Logger;

/**
 * @author Antonio Goncalves & Alexis Moussine-Pouchkine
 *         Tutorial - Beginning with The Java EE 6 Platform
 *         http://www.antoniogoncalves.org
 *         http://blogs.sun.com/alexismp
 *         --
 *         A CDI Bean with conversation scope and
 *         used as a controller by the JSF view.
 */
@Named
@ConversationScoped
public class ItemBean implements Serializable {

    // ======================================
    // =             Attributes             =
    // ======================================

    @Inject // injection point, no qualifier, used Alternatives instead
    Customer cust;

    @Inject
    Conversation conversation;

    // the scope of this attribute is set to Conversation
    List<OrderItem> order = new ArrayList<OrderItem>();

    @EJB
    private ItemEJB itemEJB;

    @EJB
    private LanguageSingleton languageSingleton;

    private Book book = new Book();
    private List<Book> bookList = new ArrayList<Book>();
    private String tags;
    @Size(min = 2, max = 2, message = "Language code should be exactly {min} characters")
    private String languageCode;

    private CD cd = new CD();
    private List<CD> cdList = new ArrayList<CD>();

    private Logger logger = Logger.getLogger("org.beginningee6.tutorial");
    
    // ======================================
    // =          Lifecycle methods         =
    // ======================================

    @PostConstruct
    private void initList() {
        bookList = itemEJB.findAllBooks();
        cdList = itemEJB.findAllCDs();
    }

    // ======================================
    // =          Business methods          =
    // ======================================

    public Boolean getForbiddenToBuy() {
        return cust.canBuy() ? Boolean.FALSE : Boolean.TRUE;
    }

    public String getBuyButtonLabel() {
        return cust.canBuy() ? "Buy me!" : "Sorry, can't buy";
    }

    public String doBuy(ActionEvent event) {
        // begin conversation on the first addition of an item
        if (conversation.isTransient()) {
            conversation.begin();
        }

        String isbn = (String) event.getComponent().getAttributes().get("isbn");
        OrderItem orderItem = cust.buy(isbn); // Will only proceed for @Premium customers
        logger.info("***** Just bought : " + orderItem);
        if (orderItem != null) {
            order.add(orderItem);
            FacesContext.getCurrentInstance().addMessage(null, new FacesMessage("Added item " + orderItem.isbn));
        }
        return null;    // Stay on the same page
    }

    public Boolean getCheckoutEnabled() {
        return conversation.isTransient() ? Boolean.TRUE : Boolean.FALSE;
    }

    public void doCheckout() {
        if (!conversation.isTransient()) {
            conversation.end(); // will flush the conversation at the end of this request
            String checkoutMessage = "Checking out Buying " + order.size() + " items: " + order;
            FacesContext.getCurrentInstance().addMessage(null, new FacesMessage(checkoutMessage));
            logger.info(checkoutMessage);
        } else {
            logger.info("No conversation in place");
            FacesContext.getCurrentInstance().addMessage(null, new FacesMessage("No conversation in place"));
        }
    }

    public String doCreateBook() {
        book.setTags(transformToList(tags));
        book.setContentLanguage(languageSingleton.getLanguageValue(languageCode));
        book = itemEJB.createBook(book);
        bookList = itemEJB.findAllBooks();
        return "newBook.xhtml";
    }

    public String doCreateCD() {
        cd = itemEJB.createCD(cd);
        cdList = itemEJB.findAllCDs();
        return "newCD.xhtml";
    }

    // ======================================
    // =           Private methods          =
    // ======================================

    private List<String> transformToList(String tagsRequestParameter) {
        List<String> listOfTags = new ArrayList<String>();
        StringTokenizer tokens = new StringTokenizer(tagsRequestParameter, ",");
        while (tokens.hasMoreElements()) {
            listOfTags.add(((String) tokens.nextElement()).trim());
        }
        return listOfTags;
    }

    // ======================================
    // =          Getters & Setters         =
    // ======================================

    public Book getBook() {
        return book;
    }

    public void setBook(Book book) {
        this.book = book;
    }

    public List<Book> getBookList() {
            return bookList;
        }

    public void setBookList(List<Book> bookList) {
        this.bookList = bookList;
    }

    public String getTags() {
        return tags;
    }

    public void setTags(String tags) {
        this.tags = tags;
    }

    public String getLanguageCode() {
        return languageCode;
    }

    public void setLanguageCode(String languageCode) {
        this.languageCode = languageCode;
    }

    public CD getCd() {
        return cd;
    }

    public void setCd(CD cd) {
        this.cd = cd;
    }

    public List<CD> getCdList() {
        return cdList;
    }

    public void setCdList(List<CD> cdList) {
        this.cdList = cdList;
    }
}
