package org.beginningee6.tutorial;

import javax.ejb.Stateless;
import javax.enterprise.context.Dependent;
import javax.inject.Inject;
import javax.interceptor.Interceptors;
import javax.persistence.EntityManager;
import javax.persistence.PersistenceContext;
import javax.persistence.Query;
import javax.ws.rs.*;
import javax.ws.rs.core.MediaType;
import java.util.List;
import javax.inject.Named;
import org.beginningee6.tutorial.order.Settlement;
import org.beginningee6.tutorial.order.SettlementMark;

/**
 * @author Antonio Goncalves & Alexis Moussine-Pouchkine
 *         Tutorial - Beginning with The Java EE 6 Platform
 *         http://www.antoniogoncalves.org
 *         http://blogs.sun.com/alexismp
 *         --
 *         A stateless EJB doing CRUD operations and queries on Books
 */
@Stateless
@Interceptors(LoggingInterceptor.class)
@Path("/items")
@Dependent
public class ItemEJB {

    // ======================================
    // =             Attributes             =
    // ======================================

    @PersistenceContext
    private EntityManager em;

    @Inject
    private IsbnGenerator isbnGenerator;

    // tezu36 Error
//    @Inject @SettlementMark
    @Inject @Named("settlement")
    private Settlement settlement;

    // ======================================
    // =          Business methods          =
    // ======================================

    @GET
    @Path("/{bookISBN}")
    @Produces({MediaType.APPLICATION_XML, MediaType.APPLICATION_JSON})
    public Book getBook(@PathParam("bookISBN") String isbn) {
        Query query = em.createNamedQuery("findBookByIsbn");
        query.setParameter("isbn", isbn);
        return (Book) query.getSingleResult();
    }

    @POST
    @Consumes({MediaType.APPLICATION_XML, MediaType.APPLICATION_JSON})
    public Book createBook(Book book) {
        book.setIsbn(isbnGenerator.generateIsbn());
        em.persist(book);
        return book;
    }

    @GET
    @Produces({MediaType.APPLICATION_XML, MediaType.APPLICATION_JSON})
    public List<Book> findAllBooks() {
        return em.createNamedQuery("findAllBooks").getResultList();
    }

    public List<Book> findAllScifiBooks() {
        return em.createNamedQuery("findAllScifiBooks").getResultList();
    }
    
    public CD createCD(CD cd) {
        em.persist(cd);
        return cd;
    }

    public List<CD> findAllCDs() {
        return em.createNamedQuery("findAllCDs").getResultList();
    }
}